@@ -640,6 +640,7 @@ module blinkled
   assign myaxi_arprot = 0;
   assign myaxi_arqos = 0;
   assign myaxi_aruser = 0;
+  reg [32-1:0] outstanding_wreq_count_0;
   reg _myaxi_read_start;
   reg [8-1:0] _myaxi_read_op_sel;
   reg [32-1:0] _myaxi_read_local_addr;
@@ -669,7 +670,7 @@ module blinkled
   reg signed [32-1:0] _th_matmul_a_addr_9;
   reg signed [32-1:0] _th_matmul_c_addr_10;
   reg signed [32-1:0] _th_matmul_i_11;
-  reg axim_flag_0;
+  reg axim_flag_1;
   reg [32-1:0] _d1_th_matmul;
   reg _th_matmul_cond_6_0_1;
   reg _myaxi_ram_a_0_read_start;
@@ -683,25 +684,25 @@ module blinkled
   reg [32-1:0] _myaxi_read_cur_global_addr;
   reg [33-1:0] _myaxi_read_cur_size;
   reg [33-1:0] _myaxi_read_rest_size;
-  reg [32-1:0] _wdata_1;
-  reg _wvalid_2;
-  reg [34-1:0] _tmp_3;
-  reg _tmp_4;
+  reg [32-1:0] _wdata_2;
+  reg _wvalid_3;
+  reg [34-1:0] _tmp_4;
+  reg _tmp_5;
   wire [32-1:0] _dataflow__variable_odata_0;
   wire _dataflow__variable_ovalid_0;
   wire _dataflow__variable_oready_0;
-  assign _dataflow__variable_oready_0 = (_tmp_3 > 0) && !_tmp_4;
+  assign _dataflow__variable_oready_0 = (_tmp_4 > 0) && !_tmp_5;
   reg _ram_a_cond_0_1;
-  reg [9-1:0] _tmp_5;
+  reg [9-1:0] counter_6;
   reg _myaxi_cond_0_1;
   assign myaxi_rready = _myaxi_read_fsm == 3;
   reg [32-1:0] _d1__myaxi_read_fsm;
   reg __myaxi_read_fsm_cond_3_0_1;
-  reg axim_flag_6;
+  reg axim_flag_7;
   reg __myaxi_read_fsm_cond_4_1_1;
   reg signed [32-1:0] _th_matmul_b_addr_12;
   reg signed [32-1:0] _th_matmul_j_13;
-  reg axim_flag_7;
+  reg axim_flag_8;
   reg _th_matmul_cond_13_1_1;
   reg _myaxi_ram_b_0_read_start;
   reg [8-1:0] _myaxi_ram_b_0_read_op_sel;
@@ -709,32 +710,32 @@ module blinkled
   reg [32-1:0] _myaxi_ram_b_0_read_global_addr;
   reg [33-1:0] _myaxi_ram_b_0_read_size;
   reg [32-1:0] _myaxi_ram_b_0_read_local_stride;
-  reg [32-1:0] _wdata_8;
-  reg _wvalid_9;
-  reg [34-1:0] _tmp_10;
-  reg _tmp_11;
+  reg [32-1:0] _wdata_9;
+  reg _wvalid_10;
+  reg [34-1:0] _tmp_11;
+  reg _tmp_12;
   wire [32-1:0] _dataflow__variable_odata_1;
   wire _dataflow__variable_ovalid_1;
   wire _dataflow__variable_oready_1;
-  assign _dataflow__variable_oready_1 = (_tmp_10 > 0) && !_tmp_11;
+  assign _dataflow__variable_oready_1 = (_tmp_11 > 0) && !_tmp_12;
   reg _ram_b_cond_0_1;
   reg __myaxi_read_fsm_cond_3_2_1;
   reg signed [32-1:0] _th_matmul_sum_14;
   reg signed [32-1:0] _th_matmul_k_15;
-  reg _tmp_12;
+  reg _tmp_13;
   reg _ram_a_cond_1_1;
   reg _ram_a_cond_2_1;
   reg _ram_a_cond_2_2;
-  reg signed [32-1:0] _tmp_13;
+  reg signed [32-1:0] _tmp_14;
   reg signed [32-1:0] _th_matmul_x_16;
-  reg _tmp_14;
+  reg _tmp_15;
   reg _ram_b_cond_1_1;
   reg _ram_b_cond_2_1;
   reg _ram_b_cond_2_2;
-  reg signed [32-1:0] _tmp_15;
+  reg signed [32-1:0] _tmp_16;
   reg signed [32-1:0] _th_matmul_y_17;
   reg _ram_c_cond_0_1;
-  reg axim_flag_16;
+  reg axim_flag_17;
   reg _th_matmul_cond_29_2_1;
   reg _myaxi_ram_c_0_write_start;
   reg [8-1:0] _myaxi_ram_c_0_write_op_sel;
@@ -747,33 +748,33 @@ module blinkled
   reg [32-1:0] _myaxi_write_cur_global_addr;
   reg [33-1:0] _myaxi_write_cur_size;
   reg [33-1:0] _myaxi_write_rest_size;
-  reg _tmp_17;
   reg _tmp_18;
-  wire _tmp_19;
+  reg _tmp_19;
   wire _tmp_20;
-  assign _tmp_20 = 1;
-  localparam _tmp_21 = 1;
-  wire [_tmp_21-1:0] _tmp_22;
-  assign _tmp_22 = (_tmp_19 || !_tmp_17) && (_tmp_20 || !_tmp_18);
-  reg [_tmp_21-1:0] __tmp_22_1;
-  wire signed [32-1:0] _tmp_23;
-  reg signed [32-1:0] __tmp_23_1;
-  assign _tmp_23 = (__tmp_22_1)? ram_c_0_rdata : __tmp_23_1;
-  reg _tmp_24;
+  wire _tmp_21;
+  assign _tmp_21 = 1;
+  localparam _tmp_22 = 1;
+  wire [_tmp_22-1:0] _tmp_23;
+  assign _tmp_23 = (_tmp_20 || !_tmp_18) && (_tmp_21 || !_tmp_19);
+  reg [_tmp_22-1:0] __tmp_23_1;
+  wire signed [32-1:0] _tmp_24;
+  reg signed [32-1:0] __tmp_24_1;
+  assign _tmp_24 = (__tmp_23_1)? ram_c_0_rdata : __tmp_24_1;
   reg _tmp_25;
   reg _tmp_26;
   reg _tmp_27;
-  reg [34-1:0] _tmp_28;
-  reg [9-1:0] _tmp_29;
+  reg _tmp_28;
+  reg [34-1:0] _tmp_29;
+  reg [9-1:0] counter_30;
   reg _myaxi_cond_1_1;
-  reg _tmp_30;
+  reg last_31;
   wire [32-1:0] _dataflow__variable_odata_2;
   wire _dataflow__variable_ovalid_2;
   wire _dataflow__variable_oready_2;
-  assign _dataflow__variable_oready_2 = (_myaxi_write_fsm == 3) && (_myaxi_write_op_sel == 1) && ((_tmp_29 > 0) && (myaxi_wready || !myaxi_wvalid));
+  assign _dataflow__variable_oready_2 = (_myaxi_write_fsm == 3) && (_myaxi_write_op_sel == 1) && ((counter_30 > 0) && (myaxi_wready || !myaxi_wvalid));
   reg _myaxi_cond_2_1;
-  assign _myaxi_write_data_done = (_tmp_30 && myaxi_wvalid && myaxi_wready)? 1 : 0;
-  reg axim_flag_31;
+  assign _myaxi_write_data_done = (last_31 && myaxi_wvalid && myaxi_wready)? 1 : 0;
+  reg axim_flag_32;
   reg [32-1:0] _d1__myaxi_write_fsm;
   reg __myaxi_write_fsm_cond_4_0_1;
   reg signed [32-1:0] _th_matmul_end_time_18;
@@ -785,7 +786,7 @@ module blinkled
   reg signed [32-1:0] _th_matmul_all_ok_24;
   reg signed [32-1:0] _th_matmul_c_addr_25;
   reg signed [32-1:0] _th_matmul_i_26;
-  reg axim_flag_32;
+  reg axim_flag_33;
   reg _th_matmul_cond_44_3_1;
   reg _myaxi_ram_c_0_read_start;
   reg [8-1:0] _myaxi_ram_c_0_read_op_sel;
@@ -793,22 +794,22 @@ module blinkled
   reg [32-1:0] _myaxi_ram_c_0_read_global_addr;
   reg [33-1:0] _myaxi_ram_c_0_read_size;
   reg [32-1:0] _myaxi_ram_c_0_read_local_stride;
-  reg [32-1:0] _wdata_33;
-  reg _wvalid_34;
-  reg [34-1:0] _tmp_35;
-  reg _tmp_36;
+  reg [32-1:0] _wdata_34;
+  reg _wvalid_35;
+  reg [34-1:0] _tmp_36;
+  reg _tmp_37;
   wire [32-1:0] _dataflow__variable_odata_4;
   wire _dataflow__variable_ovalid_4;
   wire _dataflow__variable_oready_4;
-  assign _dataflow__variable_oready_4 = (_tmp_35 > 0) && !_tmp_36;
+  assign _dataflow__variable_oready_4 = (_tmp_36 > 0) && !_tmp_37;
   reg _ram_c_cond_1_1;
   reg __myaxi_read_fsm_cond_3_3_1;
   reg signed [32-1:0] _th_matmul_j_27;
-  reg _tmp_37;
+  reg _tmp_38;
   reg _ram_c_cond_2_1;
   reg _ram_c_cond_3_1;
   reg _ram_c_cond_3_2;
-  reg signed [32-1:0] _tmp_38;
+  reg signed [32-1:0] _tmp_39;
   reg signed [32-1:0] _th_matmul_v_28;
 
   always @(posedge CLK) begin
@@ -823,39 +824,39 @@ module blinkled
   always @(posedge CLK) begin
     if(RST) begin
       ram_a_0_addr <= 0;
-      _tmp_3 <= 0;
+      _tmp_4 <= 0;
       ram_a_0_wdata <= 0;
       ram_a_0_wenable <= 0;
-      _tmp_4 <= 0;
+      _tmp_5 <= 0;
       _ram_a_cond_0_1 <= 0;
       _ram_a_cond_1_1 <= 0;
-      _tmp_12 <= 0;
+      _tmp_13 <= 0;
       _ram_a_cond_2_1 <= 0;
       _ram_a_cond_2_2 <= 0;
     end else begin
       if(_ram_a_cond_2_2) begin
-        _tmp_12 <= 0;
+        _tmp_13 <= 0;
       end 
       if(_ram_a_cond_0_1) begin
         ram_a_0_wenable <= 0;
-        _tmp_4 <= 0;
+        _tmp_5 <= 0;
       end 
       if(_ram_a_cond_1_1) begin
-        _tmp_12 <= 1;
+        _tmp_13 <= 1;
       end 
       _ram_a_cond_2_2 <= _ram_a_cond_2_1;
-      if(_myaxi_read_start && (_myaxi_read_op_sel == 1) && (_tmp_3 == 0)) begin
+      if(_myaxi_read_start && (_myaxi_read_op_sel == 1) && (_tmp_4 == 0)) begin
         ram_a_0_addr <= _myaxi_read_local_addr - _myaxi_read_local_stride;
-        _tmp_3 <= _myaxi_read_size;
+        _tmp_4 <= _myaxi_read_size;
       end 
-      if(_dataflow__variable_ovalid_0 && ((_tmp_3 > 0) && !_tmp_4) && (_tmp_3 > 0)) begin
+      if(_dataflow__variable_ovalid_0 && ((_tmp_4 > 0) && !_tmp_5) && (_tmp_4 > 0)) begin
         ram_a_0_addr <= ram_a_0_addr + _myaxi_read_local_stride;
         ram_a_0_wdata <= _dataflow__variable_odata_0;
         ram_a_0_wenable <= 1;
-        _tmp_3 <= _tmp_3 - 1;
+        _tmp_4 <= _tmp_4 - 1;
       end 
-      if(_dataflow__variable_ovalid_0 && ((_tmp_3 > 0) && !_tmp_4) && (_tmp_3 == 1)) begin
-        _tmp_4 <= 1;
+      if(_dataflow__variable_ovalid_0 && ((_tmp_4 > 0) && !_tmp_5) && (_tmp_4 == 1)) begin
+        _tmp_5 <= 1;
       end 
       _ram_a_cond_0_1 <= 1;
       if(th_matmul == 20) begin
@@ -870,39 +871,39 @@ module blinkled
   always @(posedge CLK) begin
     if(RST) begin
       ram_b_0_addr <= 0;
-      _tmp_10 <= 0;
+      _tmp_11 <= 0;
       ram_b_0_wdata <= 0;
       ram_b_0_wenable <= 0;
-      _tmp_11 <= 0;
+      _tmp_12 <= 0;
       _ram_b_cond_0_1 <= 0;
       _ram_b_cond_1_1 <= 0;
-      _tmp_14 <= 0;
+      _tmp_15 <= 0;
       _ram_b_cond_2_1 <= 0;
       _ram_b_cond_2_2 <= 0;
     end else begin
       if(_ram_b_cond_2_2) begin
-        _tmp_14 <= 0;
+        _tmp_15 <= 0;
       end 
       if(_ram_b_cond_0_1) begin
         ram_b_0_wenable <= 0;
-        _tmp_11 <= 0;
+        _tmp_12 <= 0;
       end 
       if(_ram_b_cond_1_1) begin
-        _tmp_14 <= 1;
+        _tmp_15 <= 1;
       end 
       _ram_b_cond_2_2 <= _ram_b_cond_2_1;
-      if(_myaxi_read_start && (_myaxi_read_op_sel == 2) && (_tmp_10 == 0)) begin
+      if(_myaxi_read_start && (_myaxi_read_op_sel == 2) && (_tmp_11 == 0)) begin
         ram_b_0_addr <= _myaxi_read_local_addr - _myaxi_read_local_stride;
-        _tmp_10 <= _myaxi_read_size;
+        _tmp_11 <= _myaxi_read_size;
       end 
-      if(_dataflow__variable_ovalid_1 && ((_tmp_10 > 0) && !_tmp_11) && (_tmp_10 > 0)) begin
+      if(_dataflow__variable_ovalid_1 && ((_tmp_11 > 0) && !_tmp_12) && (_tmp_11 > 0)) begin
         ram_b_0_addr <= ram_b_0_addr + _myaxi_read_local_stride;
         ram_b_0_wdata <= _dataflow__variable_odata_1;
         ram_b_0_wenable <= 1;
-        _tmp_10 <= _tmp_10 - 1;
+        _tmp_11 <= _tmp_11 - 1;
       end 
-      if(_dataflow__variable_ovalid_1 && ((_tmp_10 > 0) && !_tmp_11) && (_tmp_10 == 1)) begin
-        _tmp_11 <= 1;
+      if(_dataflow__variable_ovalid_1 && ((_tmp_11 > 0) && !_tmp_12) && (_tmp_11 == 1)) begin
+        _tmp_12 <= 1;
       end 
       _ram_b_cond_0_1 <= 1;
       if(th_matmul == 22) begin
@@ -920,35 +921,35 @@ module blinkled
       ram_c_0_wdata <= 0;
       ram_c_0_wenable <= 0;
       _ram_c_cond_0_1 <= 0;
-      __tmp_22_1 <= 0;
       __tmp_23_1 <= 0;
-      _tmp_27 <= 0;
-      _tmp_17 <= 0;
-      _tmp_18 <= 0;
-      _tmp_25 <= 0;
-      _tmp_26 <= 0;
-      _tmp_24 <= 0;
+      __tmp_24_1 <= 0;
       _tmp_28 <= 0;
-      _tmp_35 <= 0;
+      _tmp_18 <= 0;
+      _tmp_19 <= 0;
+      _tmp_26 <= 0;
+      _tmp_27 <= 0;
+      _tmp_25 <= 0;
+      _tmp_29 <= 0;
       _tmp_36 <= 0;
+      _tmp_37 <= 0;
       _ram_c_cond_1_1 <= 0;
       _ram_c_cond_2_1 <= 0;
-      _tmp_37 <= 0;
+      _tmp_38 <= 0;
       _ram_c_cond_3_1 <= 0;
       _ram_c_cond_3_2 <= 0;
     end else begin
       if(_ram_c_cond_3_2) begin
-        _tmp_37 <= 0;
+        _tmp_38 <= 0;
       end 
       if(_ram_c_cond_0_1) begin
         ram_c_0_wenable <= 0;
       end 
       if(_ram_c_cond_1_1) begin
         ram_c_0_wenable <= 0;
-        _tmp_36 <= 0;
+        _tmp_37 <= 0;
       end 
       if(_ram_c_cond_2_1) begin
-        _tmp_37 <= 1;
+        _tmp_38 <= 1;
       end 
       _ram_c_cond_3_2 <= _ram_c_cond_3_1;
       if(th_matmul == 26) begin
@@ -957,49 +958,49 @@ module blinkled
         ram_c_0_wenable <= 1;
       end 
       _ram_c_cond_0_1 <= th_matmul == 26;
-      __tmp_22_1 <= _tmp_22;
       __tmp_23_1 <= _tmp_23;
-      if((_tmp_19 || !_tmp_17) && (_tmp_20 || !_tmp_18) && _tmp_25) begin
-        _tmp_27 <= 0;
-        _tmp_17 <= 0;
+      __tmp_24_1 <= _tmp_24;
+      if((_tmp_20 || !_tmp_18) && (_tmp_21 || !_tmp_19) && _tmp_26) begin
+        _tmp_28 <= 0;
         _tmp_18 <= 0;
-        _tmp_25 <= 0;
+        _tmp_19 <= 0;
+        _tmp_26 <= 0;
       end 
-      if((_tmp_19 || !_tmp_17) && (_tmp_20 || !_tmp_18) && _tmp_24) begin
-        _tmp_17 <= 1;
+      if((_tmp_20 || !_tmp_18) && (_tmp_21 || !_tmp_19) && _tmp_25) begin
         _tmp_18 <= 1;
-        _tmp_27 <= _tmp_26;
-        _tmp_26 <= 0;
-        _tmp_24 <= 0;
-        _tmp_25 <= 1;
-      end 
-      if(_myaxi_write_start && (_myaxi_write_op_sel == 1) && (_tmp_28 == 0) && !_tmp_26 && !_tmp_27) begin
-        ram_c_0_addr <= _myaxi_write_local_addr;
-        _tmp_28 <= _myaxi_write_size - 1;
-        _tmp_24 <= 1;
-        _tmp_26 <= _myaxi_write_size == 1;
-      end 
-      if((_tmp_19 || !_tmp_17) && (_tmp_20 || !_tmp_18) && (_tmp_28 > 0)) begin
-        ram_c_0_addr <= ram_c_0_addr + _myaxi_write_local_stride;
-        _tmp_28 <= _tmp_28 - 1;
-        _tmp_24 <= 1;
-        _tmp_26 <= 0;
-      end 
-      if((_tmp_19 || !_tmp_17) && (_tmp_20 || !_tmp_18) && (_tmp_28 == 1)) begin
+        _tmp_19 <= 1;
+        _tmp_28 <= _tmp_27;
+        _tmp_27 <= 0;
+        _tmp_25 <= 0;
         _tmp_26 <= 1;
       end 
-      if(_myaxi_read_start && (_myaxi_read_op_sel == 3) && (_tmp_35 == 0)) begin
-        ram_c_0_addr <= _myaxi_read_local_addr - _myaxi_read_local_stride;
-        _tmp_35 <= _myaxi_read_size;
+      if(_myaxi_write_start && (_myaxi_write_op_sel == 1) && (_tmp_29 == 0) && !_tmp_27 && !_tmp_28) begin
+        ram_c_0_addr <= _myaxi_write_local_addr;
+        _tmp_29 <= _myaxi_write_size - 1;
+        _tmp_25 <= 1;
+        _tmp_27 <= _myaxi_write_size == 1;
       end 
-      if(_dataflow__variable_ovalid_4 && ((_tmp_35 > 0) && !_tmp_36) && (_tmp_35 > 0)) begin
+      if((_tmp_20 || !_tmp_18) && (_tmp_21 || !_tmp_19) && (_tmp_29 > 0)) begin
+        ram_c_0_addr <= ram_c_0_addr + _myaxi_write_local_stride;
+        _tmp_29 <= _tmp_29 - 1;
+        _tmp_25 <= 1;
+        _tmp_27 <= 0;
+      end 
+      if((_tmp_20 || !_tmp_18) && (_tmp_21 || !_tmp_19) && (_tmp_29 == 1)) begin
+        _tmp_27 <= 1;
+      end 
+      if(_myaxi_read_start && (_myaxi_read_op_sel == 3) && (_tmp_36 == 0)) begin
+        ram_c_0_addr <= _myaxi_read_local_addr - _myaxi_read_local_stride;
+        _tmp_36 <= _myaxi_read_size;
+      end 
+      if(_dataflow__variable_ovalid_4 && ((_tmp_36 > 0) && !_tmp_37) && (_tmp_36 > 0)) begin
         ram_c_0_addr <= ram_c_0_addr + _myaxi_read_local_stride;
         ram_c_0_wdata <= _dataflow__variable_odata_4;
         ram_c_0_wenable <= 1;
-        _tmp_35 <= _tmp_35 - 1;
+        _tmp_36 <= _tmp_36 - 1;
       end 
-      if(_dataflow__variable_ovalid_4 && ((_tmp_35 > 0) && !_tmp_36) && (_tmp_35 == 1)) begin
-        _tmp_36 <= 1;
+      if(_dataflow__variable_ovalid_4 && ((_tmp_36 > 0) && !_tmp_37) && (_tmp_36 == 1)) begin
+        _tmp_37 <= 1;
       end 
       _ram_c_cond_1_1 <= 1;
       if(th_matmul == 50) begin
@@ -1010,12 +1011,13 @@ module blinkled
     end
   end
 
-  assign _dataflow__variable_odata_2 = _tmp_23;
-  assign _dataflow__variable_ovalid_2 = _tmp_17;
-  assign _tmp_19 = 1 && _dataflow__variable_oready_2;
+  assign _dataflow__variable_odata_2 = _tmp_24;
+  assign _dataflow__variable_ovalid_2 = _tmp_18;
+  assign _tmp_20 = 1 && _dataflow__variable_oready_2;
 
   always @(posedge CLK) begin
     if(RST) begin
+      outstanding_wreq_count_0 <= 0;
       _myaxi_read_start <= 0;
       _myaxi_write_start <= 0;
       _myaxi_ram_a_0_read_start <= 0;
@@ -1033,7 +1035,7 @@ module blinkled
       myaxi_araddr <= 0;
       myaxi_arlen <= 0;
       myaxi_arvalid <= 0;
-      _tmp_5 <= 0;
+      counter_6 <= 0;
       _myaxi_cond_0_1 <= 0;
       _myaxi_ram_b_0_read_start <= 0;
       _myaxi_ram_b_0_read_op_sel <= 0;
@@ -1056,13 +1058,13 @@ module blinkled
       myaxi_awaddr <= 0;
       myaxi_awlen <= 0;
       myaxi_awvalid <= 0;
-      _tmp_29 <= 0;
+      counter_30 <= 0;
       _myaxi_cond_1_1 <= 0;
       myaxi_wdata <= 0;
       myaxi_wvalid <= 0;
       myaxi_wlast <= 0;
       myaxi_wstrb <= 0;
-      _tmp_30 <= 0;
+      last_31 <= 0;
       _myaxi_cond_2_1 <= 0;
       _myaxi_ram_c_0_read_start <= 0;
       _myaxi_ram_c_0_read_op_sel <= 0;
@@ -1080,12 +1082,18 @@ module blinkled
       if(_myaxi_cond_2_1) begin
         myaxi_wvalid <= 0;
         myaxi_wlast <= 0;
-        _tmp_30 <= 0;
+        last_31 <= 0;
+      end 
+      if(myaxi_awvalid && myaxi_awready && !(myaxi_bvalid && myaxi_bready)) begin
+        outstanding_wreq_count_0 <= outstanding_wreq_count_0 + 1;
+      end 
+      if(!(myaxi_awvalid && myaxi_awready) && (myaxi_bvalid && myaxi_bready) && (outstanding_wreq_count_0 > 0)) begin
+        outstanding_wreq_count_0 <= outstanding_wreq_count_0 - 1;
       end 
       _myaxi_read_start <= 0;
       _myaxi_write_start <= 0;
       _myaxi_ram_a_0_read_start <= 0;
-      if(axim_flag_0) begin
+      if(axim_flag_1) begin
         _myaxi_ram_a_0_read_start <= 1;
         _myaxi_ram_a_0_read_op_sel <= 1;
         _myaxi_ram_a_0_read_local_addr <= 0;
@@ -1104,24 +1112,24 @@ module blinkled
         _myaxi_read_size <= _myaxi_ram_a_0_read_size;
         _myaxi_read_local_stride <= _myaxi_ram_a_0_read_local_stride;
       end 
-      if((_myaxi_read_fsm == 2) && ((myaxi_arready || !myaxi_arvalid) && (_tmp_5 == 0))) begin
+      if((_myaxi_read_fsm == 2) && ((myaxi_arready || !myaxi_arvalid) && (counter_6 == 0))) begin
         myaxi_araddr <= _myaxi_read_cur_global_addr;
         myaxi_arlen <= _myaxi_read_cur_size - 1;
         myaxi_arvalid <= 1;
-        _tmp_5 <= _myaxi_read_cur_size;
+        counter_6 <= _myaxi_read_cur_size;
       end 
       _myaxi_cond_0_1 <= 1;
       if(myaxi_arvalid && !myaxi_arready) begin
         myaxi_arvalid <= myaxi_arvalid;
       end 
-      if(myaxi_rready && myaxi_rvalid && (_tmp_5 > 0)) begin
-        _tmp_5 <= _tmp_5 - 1;
+      if(myaxi_rready && myaxi_rvalid && (counter_6 > 0)) begin
+        counter_6 <= counter_6 - 1;
       end 
-      if(axim_flag_6) begin
+      if(axim_flag_7) begin
         _myaxi_read_idle <= 1;
       end 
       _myaxi_ram_b_0_read_start <= 0;
-      if(axim_flag_7) begin
+      if(axim_flag_8) begin
         _myaxi_ram_b_0_read_start <= 1;
         _myaxi_ram_b_0_read_op_sel <= 2;
         _myaxi_ram_b_0_read_local_addr <= 0;
@@ -1141,7 +1149,7 @@ module blinkled
         _myaxi_read_local_stride <= _myaxi_ram_b_0_read_local_stride;
       end 
       _myaxi_ram_c_0_write_start <= 0;
-      if(axim_flag_16) begin
+      if(axim_flag_17) begin
         _myaxi_ram_c_0_write_start <= 1;
         _myaxi_ram_c_0_write_op_sel <= 1;
         _myaxi_ram_c_0_write_local_addr <= 0;
@@ -1160,41 +1168,41 @@ module blinkled
         _myaxi_write_size <= _myaxi_ram_c_0_write_size;
         _myaxi_write_local_stride <= _myaxi_ram_c_0_write_local_stride;
       end 
-      if((_myaxi_write_fsm == 2) && ((myaxi_awready || !myaxi_awvalid) && (_tmp_29 == 0))) begin
+      if((_myaxi_write_fsm == 2) && ((myaxi_awready || !myaxi_awvalid) && (counter_30 == 0))) begin
         myaxi_awaddr <= _myaxi_write_cur_global_addr;
         myaxi_awlen <= _myaxi_write_cur_size - 1;
         myaxi_awvalid <= 1;
-        _tmp_29 <= _myaxi_write_cur_size;
+        counter_30 <= _myaxi_write_cur_size;
       end 
-      if((_myaxi_write_fsm == 2) && ((myaxi_awready || !myaxi_awvalid) && (_tmp_29 == 0)) && (_myaxi_write_cur_size == 0)) begin
+      if((_myaxi_write_fsm == 2) && ((myaxi_awready || !myaxi_awvalid) && (counter_30 == 0)) && (_myaxi_write_cur_size == 0)) begin
         myaxi_awvalid <= 0;
       end 
       _myaxi_cond_1_1 <= 1;
       if(myaxi_awvalid && !myaxi_awready) begin
         myaxi_awvalid <= myaxi_awvalid;
       end 
-      if(_dataflow__variable_ovalid_2 && ((_myaxi_write_fsm == 3) && (_myaxi_write_op_sel == 1) && ((_tmp_29 > 0) && (myaxi_wready || !myaxi_wvalid))) && ((_tmp_29 > 0) && (myaxi_wready || !myaxi_wvalid) && (_tmp_29 > 0))) begin
+      if(_dataflow__variable_ovalid_2 && ((_myaxi_write_fsm == 3) && (_myaxi_write_op_sel == 1) && ((counter_30 > 0) && (myaxi_wready || !myaxi_wvalid))) && ((counter_30 > 0) && (myaxi_wready || !myaxi_wvalid) && (counter_30 > 0))) begin
         myaxi_wdata <= _dataflow__variable_odata_2;
         myaxi_wvalid <= 1;
         myaxi_wlast <= 0;
         myaxi_wstrb <= { 4{ 1'd1 } };
-        _tmp_29 <= _tmp_29 - 1;
+        counter_30 <= counter_30 - 1;
       end 
-      if(_dataflow__variable_ovalid_2 && ((_myaxi_write_fsm == 3) && (_myaxi_write_op_sel == 1) && ((_tmp_29 > 0) && (myaxi_wready || !myaxi_wvalid))) && ((_tmp_29 > 0) && (myaxi_wready || !myaxi_wvalid) && (_tmp_29 > 0)) && (_tmp_29 == 1)) begin
+      if(_dataflow__variable_ovalid_2 && ((_myaxi_write_fsm == 3) && (_myaxi_write_op_sel == 1) && ((counter_30 > 0) && (myaxi_wready || !myaxi_wvalid))) && ((counter_30 > 0) && (myaxi_wready || !myaxi_wvalid) && (counter_30 > 0)) && (counter_30 == 1)) begin
         myaxi_wlast <= 1;
-        _tmp_30 <= 1;
+        last_31 <= 1;
       end 
       _myaxi_cond_2_1 <= 1;
       if(myaxi_wvalid && !myaxi_wready) begin
         myaxi_wvalid <= myaxi_wvalid;
         myaxi_wlast <= myaxi_wlast;
-        _tmp_30 <= _tmp_30;
+        last_31 <= last_31;
       end 
-      if(axim_flag_31) begin
+      if(axim_flag_32) begin
         _myaxi_write_idle <= 1;
       end 
       _myaxi_ram_c_0_read_start <= 0;
-      if(axim_flag_32) begin
+      if(axim_flag_33) begin
         _myaxi_ram_c_0_read_start <= 1;
         _myaxi_ram_c_0_read_op_sel <= 3;
         _myaxi_ram_c_0_read_local_addr <= 0;
@@ -1216,12 +1224,12 @@ module blinkled
     end
   end
 
-  assign _dataflow__variable_odata_0 = _wdata_1;
-  assign _dataflow__variable_ovalid_0 = _wvalid_2;
-  assign _dataflow__variable_odata_1 = _wdata_8;
-  assign _dataflow__variable_ovalid_1 = _wvalid_9;
-  assign _dataflow__variable_odata_4 = _wdata_33;
-  assign _dataflow__variable_ovalid_4 = _wvalid_34;
+  assign _dataflow__variable_odata_0 = _wdata_2;
+  assign _dataflow__variable_ovalid_0 = _wvalid_3;
+  assign _dataflow__variable_odata_1 = _wdata_9;
+  assign _dataflow__variable_ovalid_1 = _wvalid_10;
+  assign _dataflow__variable_odata_4 = _wdata_34;
+  assign _dataflow__variable_ovalid_4 = _wvalid_35;
   localparam th_matmul_1 = 1;
   localparam th_matmul_2 = 2;
   localparam th_matmul_3 = 3;
@@ -1305,19 +1313,19 @@ module blinkled
       _th_matmul_a_addr_9 <= 0;
       _th_matmul_c_addr_10 <= 0;
       _th_matmul_i_11 <= 0;
-      axim_flag_0 <= 0;
+      axim_flag_1 <= 0;
       _th_matmul_cond_6_0_1 <= 0;
       _th_matmul_b_addr_12 <= 0;
       _th_matmul_j_13 <= 0;
-      axim_flag_7 <= 0;
+      axim_flag_8 <= 0;
       _th_matmul_cond_13_1_1 <= 0;
       _th_matmul_sum_14 <= 0;
       _th_matmul_k_15 <= 0;
-      _tmp_13 <= 0;
+      _tmp_14 <= 0;
       _th_matmul_x_16 <= 0;
-      _tmp_15 <= 0;
+      _tmp_16 <= 0;
       _th_matmul_y_17 <= 0;
-      axim_flag_16 <= 0;
+      axim_flag_17 <= 0;
       _th_matmul_cond_29_2_1 <= 0;
       _th_matmul_end_time_18 <= 0;
       _th_matmul_time_19 <= 0;
@@ -1328,32 +1336,32 @@ module blinkled
       _th_matmul_all_ok_24 <= 0;
       _th_matmul_c_addr_25 <= 0;
       _th_matmul_i_26 <= 0;
-      axim_flag_32 <= 0;
+      axim_flag_33 <= 0;
       _th_matmul_cond_44_3_1 <= 0;
       _th_matmul_j_27 <= 0;
-      _tmp_38 <= 0;
+      _tmp_39 <= 0;
       _th_matmul_v_28 <= 0;
     end else begin
       _d1_th_matmul <= th_matmul;
       case(_d1_th_matmul)
         th_matmul_6: begin
           if(_th_matmul_cond_6_0_1) begin
-            axim_flag_0 <= 0;
+            axim_flag_1 <= 0;
           end 
         end
         th_matmul_13: begin
           if(_th_matmul_cond_13_1_1) begin
-            axim_flag_7 <= 0;
+            axim_flag_8 <= 0;
           end 
         end
         th_matmul_29: begin
           if(_th_matmul_cond_29_2_1) begin
-            axim_flag_16 <= 0;
+            axim_flag_17 <= 0;
           end 
         end
         th_matmul_44: begin
           if(_th_matmul_cond_44_3_1) begin
-            axim_flag_32 <= 0;
+            axim_flag_33 <= 0;
           end 
         end
       endcase
@@ -1393,7 +1401,7 @@ module blinkled
           end
         end
         th_matmul_6: begin
-          axim_flag_0 <= 1;
+          axim_flag_1 <= 1;
           _th_matmul_cond_6_0_1 <= 1;
           th_matmul <= th_matmul_7;
         end
@@ -1424,7 +1432,7 @@ module blinkled
           end
         end
         th_matmul_13: begin
-          axim_flag_7 <= 1;
+          axim_flag_8 <= 1;
           _th_matmul_cond_13_1_1 <= 1;
           th_matmul <= th_matmul_14;
         end
@@ -1455,27 +1463,27 @@ module blinkled
           end
         end
         th_matmul_20: begin
-          if(_tmp_12) begin
-            _tmp_13 <= ram_a_0_rdata;
+          if(_tmp_13) begin
+            _tmp_14 <= ram_a_0_rdata;
           end 
-          if(_tmp_12) begin
+          if(_tmp_13) begin
             th_matmul <= th_matmul_21;
           end 
         end
         th_matmul_21: begin
-          _th_matmul_x_16 <= _tmp_13;
+          _th_matmul_x_16 <= _tmp_14;
           th_matmul <= th_matmul_22;
         end
         th_matmul_22: begin
-          if(_tmp_14) begin
-            _tmp_15 <= ram_b_0_rdata;
+          if(_tmp_15) begin
+            _tmp_16 <= ram_b_0_rdata;
           end 
-          if(_tmp_14) begin
+          if(_tmp_15) begin
             th_matmul <= th_matmul_23;
           end 
         end
         th_matmul_23: begin
-          _th_matmul_y_17 <= _tmp_15;
+          _th_matmul_y_17 <= _tmp_16;
           th_matmul <= th_matmul_24;
         end
         th_matmul_24: begin
@@ -1498,7 +1506,7 @@ module blinkled
           th_matmul <= th_matmul_12;
         end
         th_matmul_29: begin
-          axim_flag_16 <= 1;
+          axim_flag_17 <= 1;
           _th_matmul_cond_29_2_1 <= 1;
           th_matmul <= th_matmul_30;
         end
@@ -1509,7 +1517,7 @@ module blinkled
           th_matmul <= th_matmul_32;
         end
         th_matmul_32: begin
-          if(_myaxi_write_idle) begin
+          if(_myaxi_write_idle && (outstanding_wreq_count_0 == 0)) begin
             th_matmul <= th_matmul_33;
           end 
         end
@@ -1564,7 +1572,7 @@ module blinkled
           end
         end
         th_matmul_44: begin
-          axim_flag_32 <= 1;
+          axim_flag_33 <= 1;
           _th_matmul_cond_44_3_1 <= 1;
           th_matmul <= th_matmul_45;
         end
@@ -1591,15 +1599,15 @@ module blinkled
           end
         end
         th_matmul_50: begin
-          if(_tmp_37) begin
-            _tmp_38 <= ram_c_0_rdata;
+          if(_tmp_38) begin
+            _tmp_39 <= ram_c_0_rdata;
           end 
-          if(_tmp_37) begin
+          if(_tmp_38) begin
             th_matmul <= th_matmul_51;
           end 
         end
         th_matmul_51: begin
-          _th_matmul_v_28 <= _tmp_38;
+          _th_matmul_v_28 <= _tmp_39;
           th_matmul <= th_matmul_52;
         end
         th_matmul_52: begin
@@ -1684,33 +1692,33 @@ module blinkled
       _myaxi_read_rest_size <= 0;
       _myaxi_read_cur_size <= 0;
       __myaxi_read_fsm_cond_3_0_1 <= 0;
-      _wvalid_2 <= 0;
-      _wdata_1 <= 0;
-      axim_flag_6 <= 0;
+      _wvalid_3 <= 0;
+      _wdata_2 <= 0;
+      axim_flag_7 <= 0;
       __myaxi_read_fsm_cond_4_1_1 <= 0;
       __myaxi_read_fsm_cond_3_2_1 <= 0;
-      _wvalid_9 <= 0;
-      _wdata_8 <= 0;
+      _wvalid_10 <= 0;
+      _wdata_9 <= 0;
       __myaxi_read_fsm_cond_3_3_1 <= 0;
-      _wvalid_34 <= 0;
-      _wdata_33 <= 0;
+      _wvalid_35 <= 0;
+      _wdata_34 <= 0;
     end else begin
       _d1__myaxi_read_fsm <= _myaxi_read_fsm;
       case(_d1__myaxi_read_fsm)
         _myaxi_read_fsm_3: begin
           if(__myaxi_read_fsm_cond_3_0_1) begin
-            _wvalid_2 <= 0;
+            _wvalid_3 <= 0;
           end 
           if(__myaxi_read_fsm_cond_3_2_1) begin
-            _wvalid_9 <= 0;
+            _wvalid_10 <= 0;
           end 
           if(__myaxi_read_fsm_cond_3_3_1) begin
-            _wvalid_34 <= 0;
+            _wvalid_35 <= 0;
           end 
         end
         _myaxi_read_fsm_4: begin
           if(__myaxi_read_fsm_cond_4_1_1) begin
-            axim_flag_6 <= 0;
+            axim_flag_7 <= 0;
           end 
         end
       endcase
@@ -1754,21 +1762,21 @@ module blinkled
         _myaxi_read_fsm_3: begin
           __myaxi_read_fsm_cond_3_0_1 <= 1;
           if(myaxi_rready && myaxi_rvalid && (_myaxi_read_op_sel == 1)) begin
-            _wdata_1 <= myaxi_rdata;
-            _wvalid_2 <= 1;
+            _wdata_2 <= myaxi_rdata;
+            _wvalid_3 <= 1;
           end 
           if(myaxi_rready && myaxi_rvalid && myaxi_rlast) begin
             _myaxi_read_cur_global_addr <= _myaxi_read_cur_global_addr + (_myaxi_read_cur_size << 2);
           end 
           __myaxi_read_fsm_cond_3_2_1 <= 1;
           if(myaxi_rready && myaxi_rvalid && (_myaxi_read_op_sel == 2)) begin
-            _wdata_8 <= myaxi_rdata;
-            _wvalid_9 <= 1;
+            _wdata_9 <= myaxi_rdata;
+            _wvalid_10 <= 1;
           end 
           __myaxi_read_fsm_cond_3_3_1 <= 1;
           if(myaxi_rready && myaxi_rvalid && (_myaxi_read_op_sel == 3)) begin
-            _wdata_33 <= myaxi_rdata;
-            _wvalid_34 <= 1;
+            _wdata_34 <= myaxi_rdata;
+            _wvalid_35 <= 1;
           end 
           if(myaxi_rready && myaxi_rvalid && myaxi_rlast && (_myaxi_read_rest_size > 0)) begin
             _myaxi_read_fsm <= _myaxi_read_fsm_1;
@@ -1778,7 +1786,7 @@ module blinkled
           end 
         end
         _myaxi_read_fsm_4: begin
-          axim_flag_6 <= 1;
+          axim_flag_7 <= 1;
           __myaxi_read_fsm_cond_4_1_1 <= 1;
           _myaxi_read_fsm <= _myaxi_read_fsm_5;
         end
@@ -1802,14 +1810,14 @@ module blinkled
       _myaxi_write_cur_global_addr <= 0;
       _myaxi_write_rest_size <= 0;
       _myaxi_write_cur_size <= 0;
-      axim_flag_31 <= 0;
+      axim_flag_32 <= 0;
       __myaxi_write_fsm_cond_4_0_1 <= 0;
     end else begin
       _d1__myaxi_write_fsm <= _myaxi_write_fsm;
       case(_d1__myaxi_write_fsm)
         _myaxi_write_fsm_4: begin
           if(__myaxi_write_fsm_cond_4_0_1) begin
-            axim_flag_31 <= 0;
+            axim_flag_32 <= 0;
           end 
         end
       endcase
@@ -1856,7 +1864,7 @@ module blinkled
           end 
         end
         _myaxi_write_fsm_4: begin
-          axim_flag_31 <= 1;
+          axim_flag_32 <= 1;
           __myaxi_write_fsm_cond_4_0_1 <= 1;
           _myaxi_write_fsm <= _myaxi_write_fsm_5;
         end
